@@ -18,6 +18,7 @@ import io
 import sys
 import time
 import types
+import unittest
 import warnings
 import zlib
 from collections import defaultdict
@@ -25,6 +26,7 @@ from io import StringIO
 from itertools import chain
 from random import Random
 from typing import (
+    TYPE_CHECKING,
     Any,
     BinaryIO,
     Callable,
@@ -34,6 +36,7 @@ from typing import (
     Optional,
     TypeVar,
     Union,
+    overload,
 )
 from unittest import TestCase
 
@@ -55,6 +58,7 @@ from hypothesis.errors import (
     Flaky,
     Found,
     HypothesisDeprecationWarning,
+    HypothesisException,
     HypothesisWarning,
     InvalidArgument,
     MultipleFailures,
@@ -113,7 +117,8 @@ from hypothesis.version import __version__
 
 if sys.version_info >= (3, 10):  # pragma: no cover
     from types import EllipsisType as InferType
-
+elif TYPE_CHECKING:
+    from builtins import ellipsis as InferType
 else:
     InferType = type(Ellipsis)
 
@@ -141,12 +146,15 @@ def example(*args: Any, **kwargs: Any) -> Callable[[TestFunc], TestFunc]:
     if not (args or kwargs):
         raise InvalidArgument("An example must provide at least one argument")
 
+    hypothesis_explicit_examples: List[Example] = []
+
     def accept(test):
         if not hasattr(test, "hypothesis_explicit_examples"):
-            test.hypothesis_explicit_examples = []
+            test.hypothesis_explicit_examples = hypothesis_explicit_examples
         test.hypothesis_explicit_examples.append(Example(tuple(args), kwargs))
         return test
 
+    accept.hypothesis_explicit_examples = hypothesis_explicit_examples  # type: ignore
     return accept
 
 
@@ -265,6 +273,13 @@ def is_invalid_test(test, original_argspec, given_arguments, given_kwargs):
     if not (given_arguments or given_kwargs):
         return invalid("given must be called with at least one argument")
 
+    p = inspect.signature(test).parameters
+    if p and list(p.values())[0].kind is inspect.Parameter.POSITIONAL_ONLY:
+        return invalid(
+            "given does not support tests with positional-only arguments",
+            exc=HypothesisException,
+        )
+
     if given_arguments and any(
         [original_argspec.varargs, original_argspec.varkw, original_argspec.kwonlyargs]
     ):
@@ -303,13 +318,6 @@ def is_invalid_test(test, original_argspec, given_arguments, given_kwargs):
         )
     if original_argspec.defaults or original_argspec.kwonlydefaults:
         return invalid("Cannot apply @given to a function with defaults.")
-    missing = [repr(kw) for kw in original_argspec.kwonlyargs if kw not in given_kwargs]
-    if missing:
-        return invalid(
-            "Missing required kwarg{}: {}".format(
-                "s" if len(missing) > 1 else "", ", ".join(missing)
-            )
-        )
 
     # This case would raise Unsatisfiable *anyway*, but by detecting it here we can
     # provide a much more helpful error message for people e.g. using the Ghostwriter.
@@ -549,10 +557,9 @@ class StateForActualGivenExecution:
         self.falsifying_examples = ()
         self.__was_flaky = False
         self.random = random
-        self.__warned_deadline = False
         self.__test_runtime = None
+        self.ever_executed = False
 
-        self.__had_seed = wrapped_test._hypothesis_internal_use_seed
         self.is_find = getattr(wrapped_test, "_hypothesis_internal_is_find", False)
         self.wrapped_test = wrapped_test
 
@@ -582,6 +589,7 @@ class StateForActualGivenExecution:
         swallowed the corresponding control exception.
         """
 
+        self.ever_executed = True
         data.is_find = self.is_find
 
         text_repr = None
@@ -684,7 +692,8 @@ class StateForActualGivenExecution:
                 report("Failed to reproduce exception. Expected: \n" + traceback)
             self.__flaky(
                 f"Hypothesis {text_repr} produces unreliable results: Falsified"
-                " on the first call but did not on a subsequent one"
+                " on the first call but did not on a subsequent one",
+                cause=exception,
             )
         return result
 
@@ -848,7 +857,8 @@ class StateForActualGivenExecution:
                 report(format_exception(e, e.__traceback__))
                 self.__flaky(
                     "Unreliable assumption: An example which satisfied "
-                    "assumptions on the first run now fails it."
+                    "assumptions on the first run now fails it.",
+                    cause=e,
                 )
             except BaseException as e:
                 # If we have anything for explain-mode, this is the time to report.
@@ -865,27 +875,20 @@ class StateForActualGivenExecution:
                 tb = get_trimmed_traceback()
                 report(format_exception(e, tb))
 
-            finally:  # pragma: no cover
-                # Mostly useful for ``find`` and ensuring that objects that
-                # hold on to a reference to ``data`` know that it's now been
-                # finished and they shouldn't attempt to draw more data from
-                # it.
-                ran_example.freeze()
-
-                # This section is in fact entirely covered by the tests in
-                # test_reproduce_failure, but it seems to trigger a lovely set
-                # of coverage bugs: The branches show up as uncovered (despite
-                # definitely being covered - you can add an assert False else
-                # branch to verify this and see it fail - and additionally the
-                # second branch still complains about lack of coverage even if
-                # you add a pragma: no cover to it!
-                # See https://github.com/nedbat/coveragepy/issues/623
+            finally:
+                # Whether or not replay actually raised the exception again, we want
+                # to print the reproduce_failure decorator for the failing example.
                 if self.settings.print_blob:
                     report(
                         "\nYou can reproduce this example by temporarily adding "
                         "@reproduce_failure(%r, %r) as a decorator on your test case"
                         % (__version__, encode_failure(falsifying_example.buffer))
                     )
+                # Mostly useful for ``find`` and ensuring that objects that
+                # hold on to a reference to ``data`` know that it's now been
+                # finished and they can't draw more data from it.
+                ran_example.freeze()
+
             if self.__was_flaky:
                 flaky += 1
 
@@ -903,9 +906,9 @@ class StateForActualGivenExecution:
                 f"Hypothesis found {len(self.falsifying_examples)} distinct failures."
             )
 
-    def __flaky(self, message):
+    def __flaky(self, message, *, cause):
         if len(self.falsifying_examples) <= 1:
-            raise Flaky(message)
+            raise Flaky(message) from cause
         else:
             self.__was_flaky = True
             report("Flaky example! " + message)
@@ -970,9 +973,51 @@ class HypothesisHandle:
             return self.__cached_target
 
 
+def fullargspec_to_signature(
+    argspec: inspect.FullArgSpec, *, return_annotation: object = inspect.Parameter.empty
+) -> inspect.Signature:
+    # Construct a new signature based on this argspec.  We'll later convert everything
+    # over to explicit use of signature everywhere, but this is a nice stopgap.
+
+    def as_param(name, kind, defaults):
+        annot = argspec.annotations.get(name, P.empty)
+        return P(name, kind, default=defaults.get(name, P.empty), annotation=annot)
+
+    params = []
+    P = inspect.Parameter
+    for arg in argspec.args:
+        defaults = dict(zip(argspec.args[::-1], (argspec.defaults or [])[::-1]))
+        params.append(as_param(arg, P.POSITIONAL_OR_KEYWORD, defaults))
+    if argspec.varargs:
+        params.append(as_param(argspec.varargs, P.VAR_POSITIONAL, {}))
+    for arg in argspec.kwonlyargs:
+        params.append(as_param(arg, P.KEYWORD_ONLY, argspec.kwonlydefaults or {}))
+    if argspec.varkw:
+        params.append(as_param(argspec.varkw, P.VAR_KEYWORD, {}))
+    return inspect.Signature(params, return_annotation=return_annotation)
+
+
+@overload
 def given(
-    *_given_arguments: Union[SearchStrategy, InferType],
-    **_given_kwargs: Union[SearchStrategy, InferType],
+    *_given_arguments: Union[SearchStrategy[Any], InferType],
+) -> Callable[
+    [Callable[..., Union[None, Coroutine[Any, Any, None]]]], Callable[..., None]
+]:  # pragma: no cover
+    ...
+
+
+@overload
+def given(
+    **_given_kwargs: Union[SearchStrategy[Any], InferType],
+) -> Callable[
+    [Callable[..., Union[None, Coroutine[Any, Any, None]]]], Callable[..., None]
+]:  # pragma: no cover
+    ...
+
+
+def given(
+    *_given_arguments: Union[SearchStrategy[Any], InferType],
+    **_given_kwargs: Union[SearchStrategy[Any], InferType],
 ) -> Callable[
     [Callable[..., Union[None, Coroutine[Any, Any, None]]]], Callable[..., None]
 ]:
@@ -1019,6 +1064,7 @@ def given(
         del given_arguments
 
         argspec = new_given_argspec(original_argspec, given_kwargs)
+        new_signature = fullargspec_to_signature(argspec, return_annotation=None)
 
         # Use type information to convert "infer" arguments into appropriate strategies.
         if infer in given_kwargs.values():
@@ -1029,7 +1075,7 @@ def given(
                 # not when it's decorated.
 
                 @impersonate(test)
-                @define_function_signature(test.__name__, test.__doc__, argspec)
+                @define_function_signature(test.__name__, test.__doc__, new_signature)
                 def wrapped_test(*arguments, **kwargs):
                     __tracebackhide__ = True
                     raise InvalidArgument(
@@ -1042,7 +1088,7 @@ def given(
             given_kwargs[name] = st.from_type(hints[name])
 
         @impersonate(test)
-        @define_function_signature(test.__name__, test.__doc__, argspec)
+        @define_function_signature(test.__name__, test.__doc__, new_signature)
         def wrapped_test(*arguments, **kwargs):
             # Tell pytest to omit the body of this function from tracebacks
             __tracebackhide__ = True
@@ -1172,9 +1218,17 @@ def given(
             # The next step is to use the Conjecture engine to run the test on
             # many different inputs.
 
+            ran_explicit_examples = Phase.explicit in state.settings.phases and getattr(
+                wrapped_test, "hypothesis_explicit_examples", ()
+            )
+            SKIP_BECAUSE_NO_EXAMPLES = unittest.SkipTest(
+                "Hypothesis has been told to run no examples for this test."
+            )
             if not (
                 Phase.reuse in settings.phases or Phase.generate in settings.phases
             ):
+                if not ran_explicit_examples:
+                    raise SKIP_BECAUSE_NO_EXAMPLES
                 return
 
             try:
@@ -1219,6 +1273,9 @@ def given(
                     )
                     raise the_error_hypothesis_found
 
+            if not (ran_explicit_examples or state.ever_executed):
+                raise SKIP_BECAUSE_NO_EXAMPLES
+
         def _get_fuzz_target() -> Callable[
             [Union[bytes, bytearray, memoryview, BinaryIO]], Optional[bytes]
         ]:
