@@ -56,8 +56,9 @@ from hypothesis.internal.conjecture.utils import (
 )
 from hypothesis.internal.entropy import get_seeder_and_restorer
 from hypothesis.internal.reflection import (
-    define_function_signature_from_signature,
+    define_function_signature,
     get_pretty_function_description,
+    get_signature,
     nicerepr,
     required_args,
 )
@@ -108,7 +109,8 @@ from hypothesis.utils.conventions import infer, not_set
 
 if sys.version_info >= (3, 10):  # pragma: no cover
     from types import EllipsisType as InferType
-
+elif typing.TYPE_CHECKING:  # pragma: no cover
+    from builtins import ellipsis as InferType
 else:
     InferType = type(Ellipsis)
 
@@ -118,8 +120,6 @@ try:
 except ImportError:  # < py3.8
     Protocol = object  # type: ignore[assignment]
 
-UniqueBy = Union[Callable[[Ex], Hashable], Tuple[Callable[[Ex], Hashable], ...]]
-
 
 @cacheable
 @defines_strategy()
@@ -133,18 +133,27 @@ def booleans() -> SearchStrategy[bool]:
 
 
 @overload
-def sampled_from(elements: Sequence[T]) -> SearchStrategy[T]:
-    raise NotImplementedError
+def sampled_from(elements: Sequence[T]) -> SearchStrategy[T]:  # pragma: no cover
+    ...
 
 
 @overload  # noqa: F811
-def sampled_from(elements: Type[enum.Enum]) -> SearchStrategy[Any]:
+def sampled_from(elements: Type[enum.Enum]) -> SearchStrategy[Any]:  # pragma: no cover
     # `SearchStrategy[Enum]` is unreliable due to metaclass issues.
-    raise NotImplementedError
+    ...
+
+
+@overload  # noqa: F811
+def sampled_from(
+    elements: Union[Type[enum.Enum], Sequence[Any]]
+) -> SearchStrategy[Any]:  # pragma: no cover
+    ...
 
 
 @defines_strategy(try_non_lazy=True)  # noqa: F811
-def sampled_from(elements):
+def sampled_from(
+    elements: Union[Type[enum.Enum], Sequence[Any]]
+) -> SearchStrategy[Any]:
     """Returns a strategy which generates any value present in ``elements``.
 
     Note that as with :func:`~hypothesis.strategies.just`, values will not be
@@ -205,7 +214,11 @@ def lists(
     *,
     min_size: int = 0,
     max_size: Optional[int] = None,
-    unique_by: Optional[UniqueBy] = None,
+    unique_by: Union[
+        None,
+        Callable[[Ex], Hashable],
+        Tuple[Callable[[Ex], Hashable], ...],
+    ] = None,
     unique: bool = False,
 ) -> SearchStrategy[List[Ex]]:
     """Returns a list containing values drawn from elements with length in the
@@ -389,7 +402,11 @@ def iterables(
     *,
     min_size: int = 0,
     max_size: Optional[int] = None,
-    unique_by: Optional[UniqueBy] = None,
+    unique_by: Union[
+        None,
+        Callable[[Ex], Hashable],
+        Tuple[Callable[[Ex], Hashable], ...],
+    ] = None,
     unique: bool = False,
 ) -> SearchStrategy[Iterable[Ex]]:
     """This has the same behaviour as lists, but returns iterables instead.
@@ -917,7 +934,7 @@ if sys.version_info[:2] >= (3, 8):  # pragma: no branch
     # matches the semantics of the function.  Great for documentation!
     sig = signature(builds)
     args, kwargs = sig.parameters.values()
-    builds = define_function_signature_from_signature(
+    builds = define_function_signature(
         name=builds.__name__,
         docstring=builds.__doc__,
         signature=sig.replace(
@@ -1075,8 +1092,22 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
         # The __optional_keys__ attribute may or may not be present, but if there's no
         # way to tell and we just have to assume that everything is required.
         # See https://github.com/python/cpython/pull/17214 for details.
-        optional = getattr(thing, "__optional_keys__", ())
-        anns = {k: from_type(v) for k, v in get_type_hints(thing).items()}
+        optional = set(getattr(thing, "__optional_keys__", ()))
+        anns = {}
+        for k, v in get_type_hints(thing).items():
+            origin = getattr(v, "__origin__", None)
+            if origin in types.RequiredTypes + types.NotRequiredTypes:
+                if origin in types.NotRequiredTypes:
+                    optional.add(k)
+                else:
+                    optional.discard(k)
+                try:
+                    v = v.__args__[0]
+                except IndexError:
+                    raise InvalidArgument(
+                        f"`{k}: {v.__name__}` is not a valid type annotation"
+                    ) from None
+            anns[k] = from_type(v)
         if (
             (not anns)
             and thing.__annotations__
@@ -1141,7 +1172,7 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
             )
         try:
             hints = get_type_hints(thing)
-            params = signature(thing).parameters
+            params = get_signature(thing).parameters
         except Exception:
             return builds(thing)
         kwargs = {}
@@ -1492,7 +1523,7 @@ def composite(f: Callable[..., Ex]) -> Callable[..., SearchStrategy[Ex]]:
     else:
         special_method = None
 
-    sig = signature(f)
+    sig = get_signature(f)
     params = tuple(sig.parameters.values())
 
     if not (params and "POSITIONAL" in params[0].kind.name):
@@ -1512,7 +1543,7 @@ def composite(f: Callable[..., Ex]) -> Callable[..., SearchStrategy[Ex]]:
     )
 
     @defines_strategy()
-    @define_function_signature_from_signature(f.__name__, f.__doc__, newsig)
+    @define_function_signature(f.__name__, f.__doc__, newsig)
     def accept(*args, **kwargs):
         return CompositeStrategy(f, args, kwargs)
 
@@ -1633,29 +1664,47 @@ def shared(
     return SharedStrategy(base, key)
 
 
+@composite
+def _maybe_nil_uuids(draw, uuid):
+    # Equivalent to `random_uuids | just(...)`, with a stronger bias to the former.
+    if draw(data()).conjecture_data.draw_bits(6) == 63:
+        return UUID("00000000-0000-0000-0000-000000000000")
+    return uuid
+
+
 @cacheable
 @defines_strategy(force_reusable_values=True)
-def uuids(*, version: Optional[int] = None) -> SearchStrategy[UUID]:
+def uuids(
+    *, version: Optional[int] = None, allow_nil: bool = False
+) -> SearchStrategy[UUID]:
     """Returns a strategy that generates :class:`UUIDs <uuid.UUID>`.
 
     If the optional version argument is given, value is passed through
     to :class:`~python:uuid.UUID` and only UUIDs of that version will
     be generated.
 
-    All returned values from this will be unique, so e.g. if you do
+    If ``allow_nil`` is True, generate the nil UUID much more often.
+    Otherwise, all returned values from this will be unique, so e.g. if you do
     ``lists(uuids())`` the resulting list will never contain duplicates.
 
     Examples from this strategy don't have any meaningful shrink order.
     """
+    check_type(bool, allow_nil, "allow_nil")
     if version not in (None, 1, 2, 3, 4, 5):
         raise InvalidArgument(
             f"version={version!r}, but version must be in "
             "(None, 1, 2, 3, 4, 5) to pass to the uuid.UUID constructor."
         )
-    return shared(
+    random_uuids = shared(
         randoms(use_true_random=True), key="hypothesis.strategies.uuids.generator"
     ).map(lambda r: UUID(version=version, int=r.getrandbits(128)))
 
+    if allow_nil:
+        if version is not None:
+            raise InvalidArgument("The nil UUID is not of any version")
+        return random_uuids.flatmap(_maybe_nil_uuids)
+    return random_uuids
+
 
 class RunnerStrategy(SearchStrategy):
     def __init__(self, default):
